"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MethodMeta = void 0;
const path_1 = require("path");
class MethodMeta {
    name = '';
    path = '';
    action = '';
    ref = null;
    params = [];
    constructor(name) {
        this.name = name;
    }
    bindApi(api, basePath, proto) {
        const method = this.ref;
        if (method === null) {
            throw new Error(`${this.name} method missing route decorator`);
        }
        const path = '/' + (0, path_1.join)(basePath, this.path).replace(/^\//, '');
        const types = Reflect.getMetadata("design:paramtypes", proto, this.name);
        const params = types.map((type, i) => {
            const decoratorParams = this.params[i];
            if (!decoratorParams) {
                throw new Error(`${this.name} method missing param decorator at index ${i}`);
            }
            return { ...decoratorParams, type };
        });
        console.log(`${this.action.toUpperCase()} ${path} => ${this.name}()`);
        api[this.action](path, (req, res) => {
            console.log('CALL_ROUTE', this.action, path);
            return method.apply(proto, params.map(param => {
                let value;
                if (param.getter) {
                    value = param.getter(param.data, req, res);
                }
                if (param.validator) {
                    value = param.validator.parse(value);
                }
                else if (typeof param.type === 'function') {
                    value = param.type(value);
                }
                return value;
            }));
        });
    }
}
exports.MethodMeta = MethodMeta;
