"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockGateway = void 0;
const http_1 = require("http");
const request_1 = require("./request");
const lambda_1 = require("./lambda");
const utils_1 = require("./utils");
function MockGateway(handlerFn, opts = {}) {
    const { getEvent, getContext } = (0, lambda_1.MockLambda)(opts);
    const server = (0, http_1.createServer)(handler);
    (0, utils_1.catchSigs)(terminate);
    return { listen, stop, terminate };
    async function handler(req, res) {
        try {
            let isResponse = false;
            const context = getContext();
            const event = await getEvent(req);
            const result = await handlerFn(event, context, (error) => {
                isResponse = true;
                (0, request_1.setResponse)(res, error);
            });
            isResponse || (0, request_1.setResponse)(res, result);
        }
        catch (error) {
            (0, request_1.setResponse)(res, error);
        }
        finally {
            res.end();
        }
    }
    function listen(host = '127.0.0.1', port = 3000) {
        return new Promise((resolve, reject) => server
            .on('listening', () => {
            resolve(`Server listening on ${host}:${port}`);
        })
            .on('close', () => {
            resolve(`Server closed on ${host}:${port}`);
        })
            .on('error', (err) => {
            if (err.syscall === 'listen'
                && err.code === 'EACCES') {
                reject(new Error(`Port ${port} requires elevated privileges`));
            }
            else if (err.syscall === 'listen'
                && err.code === 'EADDRINUSE') {
                reject(new Error(`Port ${port} is already in use`));
            }
            else {
                reject(err);
            }
        })
            .listen(port, host));
    }
    function stop() {
        return new Promise((resolve, reject) => {
            server.close(err => err ? reject(err) : resolve(undefined));
        });
    }
    async function terminate(value) {
        if ((0, utils_1.isError)(value)) {
            console.error(value);
        }
        try {
            await stop();
            process.kill(process.pid);
        }
        catch (err) {
            console.error(err);
        }
    }
}
exports.MockGateway = MockGateway;
