"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toString = exports.toObject = exports.isPlainObject = exports.isObject = exports.isDate = exports.isString = exports.isError = exports.isEvent = exports.catchSigs = exports.toReqTime = exports.toStreamTime = exports.numStr = exports.randBase64 = exports.randHex = void 0;
const crypto_1 = require("crypto");
const CATCH_SIGS = [
    'SIGTERM',
    'SIGUSR2',
    'uncaughtException',
    'unhandledRejection'
];
const MONTH_NAMES = [
    'Jan',
    'Feb',
    'Mar',
    'Apr',
    'May',
    'Jun',
    'Jul',
    'Aug',
    'Sep',
    'Oct',
    'Nov',
    'Dec'
];
function randHex(size) {
    return (0, crypto_1.randomBytes)(size)
        .toString('hex');
}
exports.randHex = randHex;
function randBase64(size) {
    return (0, crypto_1.randomBytes)(size)
        .toString('base64');
}
exports.randBase64 = randBase64;
function numStr(num) {
    return num < 10
        ? `0${num}`
        : `${num}`;
}
exports.numStr = numStr;
function toStreamTime(date) {
    return date.getFullYear() + '/' +
        numStr(date.getMonth() + 1) + '/' +
        numStr(date.getDate());
}
exports.toStreamTime = toStreamTime;
function toReqTime(date) {
    return numStr(date.getDate()) + '/' +
        MONTH_NAMES[date.getMonth()] + '/' +
        date.getFullYear() + ':' +
        numStr(date.getHours()) + ':' +
        numStr(date.getMinutes()) + ':' +
        numStr(date.getSeconds()) + ' +0000';
}
exports.toReqTime = toReqTime;
function catchSigs(cb) {
    CATCH_SIGS.forEach(event => process.once(event, cb));
}
exports.catchSigs = catchSigs;
function isEvent(value) {
    return isPlainObject(value)
        && typeof value.statusCode === 'number';
}
exports.isEvent = isEvent;
function isError(value) {
    return value instanceof Error;
}
exports.isError = isError;
function isString(value) {
    return typeof value === 'string';
}
exports.isString = isString;
function isDate(value) {
    return value
        && Object.prototype.toString.call(value) === "[object Date]"
        && !isNaN(value);
}
exports.isDate = isDate;
function isObject(value) {
    return value && typeof value === 'object';
}
exports.isObject = isObject;
function isPlainObject(value) {
    return value
        && Object.prototype.toString.call(value) === '[object Object]';
}
exports.isPlainObject = isPlainObject;
function toObject(value) {
    return isObject(value) ? value : {};
}
exports.toObject = toObject;
function toString(value) {
    if (isString(value)) {
        return value;
    }
    if (isError(value)) {
        return value.toString();
    }
    if (isDate(value)) {
        return value.toISOString();
    }
    if (value instanceof Buffer) {
        return value.toString('utf8');
    }
    return JSON.stringify(value);
}
exports.toString = toString;
